<?php
declare(strict_types=1);

namespace Wildix\Calendar;

use DateTime;
use DateTimeZone;
use Wildix\Colleagues\Colleagues;
use Wildix\Ejabberd;
use Wildix\Exception;
use Logger\Log;

class SyncEventsToJabber {
    public function __construct(
        private Events $events,
        private Colleagues $colleagues,
        private Ejabberd $jabber,
        private Log $log,
        private int $appId
    ) {
    }

    public function run(string $time): void {
        $ids    = [];
        $events = $this->eventsService()->getEventsByTime($time, $this->appId);

        foreach ($events->groupBy('owner') as $userId => $items) {
            try {
                $items      = $items->toAsArrays();
                $freeStatus = empty(array_filter($items, fn($item) => (bool)$item['processed']));
                if ($freeStatus) {
                    $event = $this->selectFirstEvent($items);
                    $ids[] = $this->setEvent($userId, $event);
                }
            } catch (\Exception $e) {
                $message = sprintf('Failed to set presence for user::%s, message::%s', $userId, $e->getMessage());
                $this->getLogger()->error(__LINE__, $message);
            }
        }
        $this->eventsService()->setProcessed($ids, 1);
        $this->eventsService()->deleteEvents(['dateEnd' => ['to' => $time]]);
    }

    private function setEvent(int $userId, array $event): string {
        $user = $this->colleaguesService()->getUserByID($userId);
        if (!$user) {
            throw new Exception('User not found');
        }
        $settings = $user->getCalendarSettings()['OUTLOOK'];

        if ($settings['enable'] == 0 && $this->appId !== Events::EVENT_APP_OFFICE365_OUTLOOK) {
            throw new Exception(sprintf('Enable::%s, appId::%s', $settings['enable'], $this->appId));
        }

        if ((int)$event['type'] === Events::EVENT_TYPE_WORKING_HOURS && (int)$settings['working_hours']['enable'] === 0) {
            throw new Exception(sprintf('Type::%s, workingHours::%s', $event['type'], $settings['working_hours']['enable']));
        }

        $status = $this->eventsService()->convertEventToStatus($event, $settings);
        $until  = $status['until'] ? $status['until']->format('d/m/Y H:i') : '';
        $this->jabberService()->setUserPresence($user->getExtension(), $status['status'], $status['message'], $until);

        return $event['id'];
    }

    private function selectFirstEvent($events): array {
        $timeZone = new DateTimeZone('UTC');
        $events   = array_map(
            function ($item) use ($timeZone) {
                $item['dateStart'] = DateTime::createFromFormat('Y-m-d\TH:i:00\Z', $item['dateStart'], $timeZone);
                $item['dateEnd']   = DateTime::createFromFormat('Y-m-d\TH:i:00\Z', $item['dateEnd'], $timeZone);

                return $item;
            },
            $events
        );

        usort($events, fn ($a, $b) => $a['dateStart'] < $b['dateStart'] ? -1 : 1);

        return $events[0];
    }

    private function getLogger(): Log {
        return $this->log;
    }

    private function eventsService(): Events {
        return $this->events;
    }

    private function jabberService(): Ejabberd {
        return $this->jabber;
    }

    private function colleaguesService(): Colleagues {
        return $this->colleagues;
    }
}
